//
//  AreteEarphoneRfidViewController.m
//  AreteEarphoneRfid
//
//  Created by Asterisk on 13. 1. 25.
//  Copyright (c) 2013 Asterisk. All rights reserved.
//
#import "Utility.h"
#import "AreteRfidViewController.h"
#import "TagViewController.h"
#import "InfoViewController.h"
#import "settingsController.h"
#import "RcpRfidApi.h"
#import <AudioToolBox/AudioServices.h>
#import <CoreTelephony/CTCallCenter.h>
#import <CoreTelephony/CTCall.h>
#import "Indicator.h"

@interface AreteRfidViewController()
{
    int batteryLevelArray[10];
    int batteryCounter;
    BOOL charging;
    BOOL plugged;
    CTCallCenter *callCenter;
    BOOL tagRssiOn;
    CFURLRef		soundFileURLRef;
	SystemSoundID	soundFileObject;
}
@property (nonatomic, strong, readwrite) TagViewController *tagViewController;
@property (nonatomic, strong, readwrite) InfoViewController *infoViewController;
@property (nonatomic, strong, readwrite) SettingsController *settingsController;
@property (nonatomic, strong, readwrite) RcpRfidApi *rcp;
@property (nonatomic,assign) int count;
@end

@implementation AreteRfidViewController

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"segueTagTableView"])
    {
        self.tagViewController = segue.destinationViewController;
        self.tagViewController.rcp = self.rcp;
    }
    else if ([segue.identifier isEqualToString:@"segueSettings"])
    {
        [self.rcp stopReadTags];
        self.settingsController = segue.destinationViewController;
        self.settingsController.rcp = self.rcp;
        self.olBtnRead.enabled = YES;
    }
}

- (void)appWillTerminate:(NSNotification*) notification
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    NSLog(@"appWillTerminate\n");
}

- (void)appDidEnterBackground:(NSNotification*) notification
{
    self.olBtnRead.enabled = NO;
    self.olBtnClear.enabled = NO;
    self.olBtnStop.enabled = NO;
    self.olBtnSettings.enabled = NO;
    
    if(self.olSwitch.isOn)
    {
        [self.olSwitch setOn:NO];
    }
    
    NSLog(@"appDidEnterBackground\n");
}

#pragma mark - View lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    NSLog(@"viewDidLoad\n");
    
    [[NSNotificationCenter defaultCenter]
        addObserver:self
        selector:@selector(appDidEnterBackground:)
        name:UIApplicationDidEnterBackgroundNotification
        object:nil];
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(appDidEnterBackground:)
     name:UIApplicationWillResignActiveNotification
     object:nil];
    
    [[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(appWillTerminate:)
     name:UIApplicationWillTerminateNotification
     object:nil];
    
    if(!callCenter)
    {
        _rcp = self.rcp;
        __weak RcpRfidApi *weakRcp = _rcp;
        __weak AreteRfidViewController *weakSelf = self;
        
        callCenter = [[CTCallCenter alloc]init];
        callCenter.callEventHandler = ^(CTCall *call)
        {
            NSLog(@"call.callState = %@", call.callState);
            if((call.callState == CTCallStateIncoming)
               || (call.callState == CTCallStateDialing)
               || (call.callState == CTCallStateConnected)
               || (call.callState == CTCallStateDisconnected))
            {
                RcpRfidApi *strongRcp = weakRcp;
                if(strongRcp)
                {
                    if([strongRcp isOpened])
                        [strongRcp close];
                }
                
                dispatch_async(dispatch_get_main_queue(),
                ^{
                    AreteRfidViewController *strongSelf = weakSelf;
                    if(strongSelf)
                    {
                        [strongSelf displayClose];
                        [strongSelf.olSwitch setOn:NO];
                    }
                });
            }
        };
    }
    NSLog(@"appDidFinishLaunching\n");
    
    NSDictionary *defaults
    = [NSDictionary dictionaryWithObjectsAndKeys:
       @"0", @"stopTagCount",
       @"0", @"stopTime",
       @"0", @"stopCycle",
       @"0", @"tagRssi",
       @"1", @"beep",
       nil];
    
    [[NSUserDefaults standardUserDefaults] registerDefaults:defaults];
    self.olBtnSettings.enabled = NO;
    
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    NSLog(@"viewWillAppear\n");
    
    BOOL rssiOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"tagRssi"];

    if(tagRssiOn != rssiOn)
    {
        [self.tagViewController.tagIDArray removeAllObjects];
        [self.tagViewController.tagCountArray removeAllObjects];
        [self.tagViewController.tableView reloadData];
        self.olTagCount.text = @"0";
        
        tagRssiOn = rssiOn;
    }
    
    
    NSURL *tapSound   = [[NSBundle mainBundle] URLForResource: @"read"
                                                withExtension: @"caf"];
    // Store the URL as a CFURLRef instance
    soundFileURLRef = (__bridge CFURLRef) tapSound;
    
    // Create a system sound object representing the sound file.
    AudioServicesCreateSystemSoundID (soundFileURLRef,
                                      &soundFileObject);

    
#ifdef __DEBUG_VIEW__
    self.olStatusTextView.hidden = NO;
    self.olStatusTextView.text = @"";
#else
    self.olStatusTextView.hidden = YES;
#endif
    
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    NSLog(@"viewDidAppear");
    
    self.rcp.delegate = self;
}


- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
    NSLog(@"viewWillDisappear");
    
    AudioServicesDisposeSystemSoundID (soundFileObject);
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
    NSLog(@"viewDidDisappear");
}

- (RcpRfidApi *)rcp
{
    static dispatch_once_t pred = 0;
    __strong static id _sharedObject = nil;
    dispatch_once(&pred,
                  ^{
                      _sharedObject = [[RcpRfidApi alloc] init];
                      _rcp = _sharedObject;
                      _rcp.delegate = self;
                  });
    return _sharedObject;
}

- (void)pcEpcReceived:(NSData*)pcEpc
{
    NSLog(@"pcEpcReceived");
    dispatch_async(dispatch_get_main_queue(),
    ^{
        if(![self.tagViewController.tagIDArray containsObject:pcEpc])
        {
            int tagCount = (int)([self.tagViewController.tagCountArray count] + 1);
            [self.tagViewController.tagCountArray addObject:[NSNumber numberWithInt:1]];
            [self.tagViewController.tagIDArray addObject:pcEpc];
            self.olTagCount.text = [NSString stringWithFormat:@"%d",tagCount];
            
            BOOL beep = [[NSUserDefaults standardUserDefaults] boolForKey:@"beep"];
            
            if(beep)
            {
                AudioServicesPlaySystemSound (soundFileObject);
            }
        }
        else
        {
            int index = (int)[self.tagViewController.tagIDArray indexOfObject:pcEpc];
            int count = (int)[[self.tagViewController.tagCountArray objectAtIndex:index] integerValue];
            [self.tagViewController.tagCountArray
             replaceObjectAtIndex:index withObject:[NSNumber numberWithInt:(count + 1)]];
        }
        [self.tagViewController.tableView reloadData];
	});
}

- (void)pcEpcRssiReceived:(NSData*)pcEpc rssi:(int8_t)rssi
{
    dispatch_async(dispatch_get_main_queue(),
    ^{
       if(![self.tagViewController.tagIDArray containsObject:pcEpc])
       {
           int tagCount = (int)([self.tagViewController.tagCountArray count] + 1);
           [self.tagViewController.tagCountArray addObject:[NSNumber numberWithInt:(rssi)]];
           [self.tagViewController.tagIDArray addObject:pcEpc];
           self.olTagCount.text = [NSString stringWithFormat:@"%d",tagCount];
           
           BOOL beep
           = [[NSUserDefaults standardUserDefaults] boolForKey:@"beep"];
           
           if(beep)
           {
               AudioServicesPlaySystemSound (soundFileObject);
           }
       }
       else
       {
           int index = (int)[self.tagViewController.tagIDArray indexOfObject:pcEpc];
           [self.tagViewController.tagCountArray
            replaceObjectAtIndex:index withObject:[NSNumber numberWithInt:(rssi)]];
       }    	
       [self.tagViewController.tableView reloadData];
    });    
}

- (void)readerConnected{
    [self readerConnected:0xff];
}

- (void)readerConnected:(uint8_t)status
{
    NSLog(@"readerConnected\n");
    
    
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       switch (status)
                       {
                           case 0xff:
                           {
                               self.olStatus.text = @"Connected\n";
                               [self.olSwitch setEnabled:YES];
                               [self.olSwitch setOn:YES];
                               self.olBtnRead.enabled = YES;
                               self.olBtnClear.enabled = YES;
                               self.olBtnStop.enabled = YES;
                               self.olBtnSettings.enabled = YES;
                               
                               NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
                               BOOL beep = [defaults boolForKey:@"beep"];
                               BOOL Vibration = [defaults boolForKey:@"viberation"];
                               
                               [self.rcp setBeep:(uint8_t)beep setVibration:(uint8_t)Vibration setIllumination:(uint8_t)0x00];
                               
                               [Indicator show:NO target:self];

                           }
                               break;
                            case 0x00:
                               break;
                       }
                   });
}

- (void)startedReadTags:(uint8_t)status;
{
    switch (status)
    {
        case 0x00:
        {
            NSLog(@"Read Tags Started\n");
            dispatch_async(dispatch_get_main_queue(),
            ^{
                self.olBtnRead.enabled = NO;
            });
        }
            break;
        case 0x1F:
        {
            NSLog(@"Read Tags Completed\n");
            dispatch_async(dispatch_get_main_queue(),
            ^{
                self.olBtnRead.enabled = YES;
            });
        }
            break;
        default:
            break;
    }
}

- (void)stoppedReadTags:(uint8_t)statusCode{
    
    if (statusCode == 0x00) {
        dispatch_async(dispatch_get_main_queue(), ^{
            self.olBtnRead.enabled = YES;
        });
    }
}

- (void)batteryChargeReceived:(int)battery{
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       self.olBattery.text = [NSString stringWithFormat:@"%d%% ", battery];
                   });
}

- (void)pluggedRfid:(BOOL)plug
{
    plugged = plug;
    
    NSLog(@"[@%@]Plugged = %d\n",self.description, plug);
    
    if(plug)
    {

        if([self.rcp isOpened]){
            dispatch_async(dispatch_get_main_queue(),^{
                self.olStatus.text = @"Plugged";
                [self.olSwitch setEnabled:YES];
            });
        }
    }else{
        dispatch_async(dispatch_get_main_queue(),^{
            self.olStatus.text = @"Unplugged";
            [self.rcp close];
            [self displayClose];
        });
    }
}

- (IBAction)actionSwitch:(UISwitch *)sender
{
    NSLog(@"actionSwitch:%d\n", [sender isOn]);
    if([sender isOn])
    {
        if([self.rcp open])
        {
            [self.rcp setReaderPower:YES];
            
            [Indicator show:YES target:self];
            
            self.olBtnRead.enabled = YES;
            self.olBtnClear.enabled = YES;
            self.olBtnStop.enabled = YES;
        }
    }
    else
    {
        if([self.rcp isOpened])
        {
            [self.rcp setReaderPower:NO];
        }
       self.olBtnSettings.enabled = NO;
         [Indicator show:NO target:self];
        [self.rcp close];
        [self displayClose];       
    }
}

- (void)displayClose
{
    NSLog(@"displayClose\n");
    
    self.olBtnRead.enabled = NO;
    self.olBtnClear.enabled = NO;
    self.olBtnStop.enabled = NO;
    self.olBtnSettings.enabled = NO;
    
    if(plugged)
    {
        self.olStatus.text = @"Plugged";
    }
    else
    {
        self.olStatus.text = @"Unplugged";
        [self.olSwitch setOn:NO];
        self.olSwitch.enabled = NO;
    }
    
    self.olBattery.text = @"0% "; 
}

- (IBAction)btnRead:(UIBarButtonItem *)sender
{
    NSString *text = [sender title];
    NSLog(@"%@\n",text);
	
    if(![self.rcp isOpened])
        return;

    int stopTagCount = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"stopTagCount"];
    int stopTime = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"stopTime"];
    int stopCycle = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"stopCycle"];
    BOOL rssiOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"tagRssi"];
    
    if(!rssiOn)
    {
        [self.rcp startReadTags:stopTagCount mtime:stopTime repeatCycle:stopCycle];
    }
    else
    {
        [self.rcp startReadTagsWithRssi:stopTagCount mtime:stopTime repeatCycle:stopCycle];
    }
}

- (IBAction)btnClear:(UIBarButtonItem *)sender 
{
    NSString *text = [sender title];
    NSLog(@"%@\n",text);    
    [self.tagViewController.tagIDArray removeAllObjects];
    [self.tagViewController.tagCountArray removeAllObjects];
    [self.tagViewController.tableView reloadData];
    self.olTagCount.text = @"0";
#ifdef __DEBUG_VIEW__
    self.olStatusTextView.text = @"";
#endif
    
    self.countLabel.text = @"0";
}

- (IBAction)btnStop:(UIBarButtonItem *)sender 
{    
    NSString *text = [sender title];
    NSLog(@"%@\n",text);
       
    if(![self.rcp isOpened])
        return;

    [self.rcp stopReadTags];
    
    self.olBtnRead.enabled = YES;
}


@end

