//
//  SettingPopController.m
//  AretePop
//
//  Created by Asterisk on 13. 6. 17..
//
//

#import "SettingsPopController.h"
#import "SettingsPowerController.h"
#import "SettingsFhLbtController.h"
#import "AntiColViewController.h"
#import "SessionViewController.h"
#import "Utility.h"
#import "SettingsStopController.h"
#import "Indicator.h"


@interface SettingsPopController ()
{
    Boolean beep;
    int16_t power1;
    int16_t powerMin;
    int16_t powerMax;
    int16_t onTime;
    int16_t offTime;
    int16_t senseTime;
    int16_t lbtLevel;
    Boolean fhEnable;
    Boolean lbtEnable;
    Boolean cwEnable;
    int16_t readTime;
    int16_t idleTime;
    int16_t cst;
    int16_t rfl;
    uint8_t lbt;
    uint8_t cw;
    uint8_t session;
    uint8_t antiColMode;
    uint8_t startQ;
    uint8_t minQ;
    uint8_t maxQ;
    
    uint8_t tagCount;
    uint8_t time;
    uint16_t rCycle;

}
@property(nonatomic, strong, readwrite) SettingsPowerController *settingsPowerController;
@property(nonatomic, strong, readwrite) SettingsFhLbtController *settingsFhLbtController;
@property(nonatomic, strong, readwrite) AntiColViewController *antiColController;
@property(nonatomic, strong, readwrite) SessionViewController *sessionController;
@property(nonatomic, strong, readwrite) SettingsStopController *settingsStopController;
@property (weak, nonatomic) IBOutlet UISwitch *olHopping;
@property (weak, nonatomic) IBOutlet UILabel *sessionLabel;
@end

@implementation SettingsPopController
@synthesize rcp = _rcp;
@synthesize settingsPowerController = _settingsPowerController;
@synthesize settingsFhLbtController = _settingsFhLbtController;
@synthesize settingsStopController = _settingsStopController;

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([segue.identifier isEqualToString:@"seguePowerSettings"])
    {
        _settingsPowerController = segue.destinationViewController;
        _settingsPowerController.rcp = _rcp;
        for(int i = powerMin; i <= powerMax; i = i + 5)
        {
            [_settingsPowerController.powerArray addObject:[NSNumber numberWithInt:i]];
        }
        _settingsPowerController.initPower = power1;
    }
    else if ([segue.identifier isEqualToString:@"seguefhLbtSettings"])
    {
        _settingsFhLbtController = segue.destinationViewController;
        _settingsFhLbtController.rcp = _rcp;

        _settingsFhLbtController.onTime = readTime;
        _settingsFhLbtController.offTime = idleTime;
        _settingsFhLbtController.senseTime = senseTime;
        _settingsFhLbtController.lbtLevel = lbtLevel;
        _settingsFhLbtController.fhEnable = fhEnable;
        _settingsFhLbtController.lbtEnable = lbtEnable;
        _settingsFhLbtController.cwEnable = cwEnable;
    }
    else if ([segue.identifier isEqualToString:@"segueAntiCollision"])
    {
        _antiColController = segue.destinationViewController;
        _antiColController.rcp = _rcp;
        _antiColController.mode = antiColMode;
        _antiColController.start = startQ;
        _antiColController.max = maxQ;
        _antiColController.min = minQ;
    }
    else if ([segue.identifier isEqualToString:@"segueSession"])
    {
        _sessionController = segue.destinationViewController;
        _sessionController.rcp = _rcp;
        _sessionController.session = session;
    }
    else if ([segue.identifier isEqualToString:@"segueStopSettings"])
    {
        _settingsStopController = segue.destinationViewController;
        _settingsStopController.rcp = _rcp;
        _settingsStopController.tagCount = tagCount;// 151005 PYH
        _settingsStopController.time = time;// 151005 PYH
        _settingsStopController.cycle = rCycle;// 151005 PYH
    }
}

- (void)viewDidLoad
{
    [super viewDidLoad];
}

- (void)viewWillAppear:(BOOL)animated
{
    printf("viewWillAppear\n");
    
    [super viewWillAppear:animated];
    
    int stopTagCount = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"stopTagCount"];
    int stopTime = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"stopTime"];
    int stopCycle = (int)[[NSUserDefaults standardUserDefaults] integerForKey:@"stopCycle"];
    
    self.olStopCondition.text
    = [NSString stringWithFormat:@"%d/%d/%d",
       stopTagCount,
       stopTime,
       stopCycle];
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSInteger selectedMode = [defaults integerForKey:@"anti_col_mode"];
    self.olAntiCollisionMode.text = [NSString stringWithFormat:@"%ld",(long)selectedMode];
    
    NSArray *params = @[@"S0",@"S1",@"S2",@"S3"];
    NSInteger sessionNo = [defaults integerForKey:@"session"];
    self.olSession.text = params[sessionNo];
    
    BOOL rssiOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"tagRssi"];
    
    [self.olTagRssi setOn:rssiOn];
    
    self.rcp.delegate = self;

   [self.rcp getOutputPowerLevel];
}

- (void)sessionReceived:(uint8_t)session_{
    
    dispatch_async(dispatch_get_main_queue(), ^{
        self.sessionLabel.text = [NSString stringWithFormat:@"%d",(int)session_];
        [self.rcp getChannel];
    });

    session = session_;
}

- (void)anticolParamReceived:(uint8_t)mode start:(uint8_t)start max:(uint8_t)max min:(uint8_t)min{
    dispatch_async(dispatch_get_main_queue(), ^{
        
        [self.rcp getSession];
        
        antiColMode = mode;
        startQ = start;
        maxQ = max;
        minQ = min;
    });
}

- (void)regionReceived:(uint8_t)region{
    if (region == 0x41) {
        powerMax = 240;
        if (power1 > 240) {
            power1 = 240;
        }
    }
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)didSetOutputPowerLevel:(uint8_t)status{
    
    switch (status) {
        case 0x00:

            dispatch_async(dispatch_get_main_queue(), ^{
                
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"Updated." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
                [alert show];
            });
            [self.rcp updateRegistry];
            break;
            
        default:
            break;
    }
}

- (void)didSetSession:(uint8_t)status{
    switch (status) {
        case 0x00:
            
            dispatch_async(dispatch_get_main_queue(), ^{
                
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"Updated." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
                [alert show];
            });
            [self.rcp updateRegistry];
            break;
            
        default:
            break;
    }
}

- (void)didSetFhLbtReceived:(uint8_t)statusCode{
    switch (statusCode) {
        case 0x00:
            
            dispatch_async(dispatch_get_main_queue(), ^{
                
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"Updated." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
                [alert show];
            });
            [self.rcp updateRegistry];
            break;
            
        default:
            break;
    }
}

- (void)didsetStopConditionMtnu:(uint8_t)statusCode{
    
    dispatch_async(dispatch_get_main_queue(), ^{
        
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:[NSString stringWithFormat:@"0X%02X",statusCode] delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
        [alert show];
    });
    
    switch (statusCode) {
        case 0x00:
            
            dispatch_async(dispatch_get_main_queue(), ^{
                
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"Updated." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
                [alert show];
            });
            [self.rcp updateRegistry];
            break;
            
        default:
            break;
    }
}

- (void)didSetAntiColModeReceived:(uint8_t)statusCode{
    switch (statusCode) {
        case 0x00:
            
            dispatch_async(dispatch_get_main_queue(), ^{
                
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil message:@"Updated." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
                [alert show];
            });
            [self.rcp updateRegistry];
            
            break;
            
        default:
            break;
    }
    
}

- (void)channelReceived:(uint8_t)channel channelOffset:(uint8_t)channelOffset{
    dispatch_async(dispatch_get_main_queue(), ^{
        self.olChannelLabel.text = [NSString stringWithFormat:@"%d",channel];
        [self.rcp getFhLbtParam];
    });
}

- (void)frequencyHoppingModeReceived:(uint8_t)statusCode{
    
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       switch (statusCode) {
                           case 0x00:
                               //Normal mode
                               [self.olHopping setOn:NO];
                               break;
                               
                           case 0x01:
                               // Smart Hopping mode
                               [self.olHopping setOn:YES];
                               break;
                           default:
                               break;
                       }
                       
                   });
}

- (void)registeryItemReceived:(NSData *)item{
    dispatch_async(dispatch_get_main_queue(), ^{
        self.olRssiOffsetLabel.text = [item bytesToHexString];
    });
}

- (void)txPowerLevelReceived:(NSData*)power{
    dispatch_async(dispatch_get_main_queue(), ^{
        
        Byte *b = (Byte*)[power bytes];
        
        power1 = (b[0] << 8) | b[1];
        powerMin = (b[2] << 8) | b[3];
        powerMax = (b[4] << 8) | b[5];
        
        self.olPowerLevel.text = [NSString stringWithFormat:@"%02.1f", (float)power1/10];
        
        [self.rcp getAnticollision];
        
    });
}

- (IBAction)eHoppingChanged:(id)sender {
    
    NSString* alertString = @"";
    
    if ([self.olHopping isOn]) {
        alertString = @"This process will choose optimum channel for operation.\nFor proper poeration, don't block antenna during process.\nIt takes up to 10 seconds";
    }else{
        alertString = @"Normal mode feature disables smart hopping feature";
    }
    
    dispatch_async(dispatch_get_main_queue(),^{
        UIAlertView *selectHPMode = [[UIAlertView alloc] initWithTitle:@"Hopping"
                                                               message:alertString
                                                              delegate:self
                                                     cancelButtonTitle:NSLocalizedString(@"OK", @"")
                                                     otherButtonTitles:nil];
        
        [selectHPMode show];
        
    });
}

- (void) alertView:(UIAlertView *)selectHPMode clickedButtonAtIndex:(NSInteger)buttonIndex{
    
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       [Indicator show:YES target:self];
                   });
    
    if ([self.olHopping isOn]) {
        [self.rcp setOptimumFrequencyHoppingTable];
    }else{
        [self.rcp SetFrequencyHoppingMode:0x00];
    }
}

- (void)didSetFreqHPMode:(uint8_t)status{
    
    if (status == 0x00) {
        dispatch_async(dispatch_get_main_queue(),
                       ^{
                           [Indicator show:NO target:self];
                           [self.rcp updateRegistry];
                       });
    }
}

- (void)didSetFreqHPTable:(uint8_t)status{
    
    if (status == 0x00) {
        [self.rcp updateRegistry];
        dispatch_async(dispatch_get_main_queue(),
                       ^{
                           [Indicator show:NO target:self];
                       });
    }
}

- (void)didSetOptiFreqHPTable:(uint8_t)status{
    
    if (status == 0x01) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.rcp SetFrequencyHoppingMode:0x01];
        });
    }
}

- (void)errDetailReceived:(NSData *)errCode{
    
    Byte *b = (Byte*) [errCode bytes];
    dispatch_async(dispatch_get_main_queue(),
                   ^{
                       switch (b[1]) {
                           case 0xE4:
                           case 0xE6:
                           case 0xD2:
                               [Indicator show:NO target:self];
                               break;
                               
                           default:
                               break;
                       }
                   });
}


#pragma mark - Table view delegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.section == 2) {
        if (indexPath.row == 2) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.rcp setGpadc];
            });
        }else if (indexPath.row == 3){
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.rcp setChannel:0x18 channelOffset:0x00];
                [self.rcp getChannel];
            });
        }
    }

}

- (void)viewDidUnload
{
    [self setOlPowerLevel:nil];
    [self setOlOnOffTime:nil];
    [self setOlBeep:nil];
    [self setOlStopCondition:nil];
    [super viewDidUnload];
}

- (IBAction)eBeepChanged:(id)sender
{    
    [self.rcp setBeep:(uint8_t)self.olBeep.on setVibration:self.olVibe.on setIllumination:self.olBeep.on];
    [[NSUserDefaults standardUserDefaults] setBool:self.olBeep.isOn forKey:@"beep"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    dispatch_async(dispatch_get_main_queue(), ^{
        
    });
}
- (IBAction)eVibeChanged:(id)sender {
    [self.rcp setBeep:(uint8_t)self.olBeep.on setVibration:self.olVibe.on setIllumination:self.olBeep.on];
    [[NSUserDefaults standardUserDefaults] setBool:self.olVibe.isOn forKey:@"viberation"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    dispatch_async(dispatch_get_main_queue(), ^{
        
    });
}

- (void)fhLbtReceived:(NSData *)fhLb{
    Byte *b = (Byte*) [fhLb bytes];
    
    if (fhLb.length < 11) return;
    
    dispatch_async(dispatch_get_main_queue(), ^{

        
        if (b[8] == 0x00) {
            [self.olFh setOn:NO];
        }else{
            [self.olFh setOn:YES];
        }
        readTime = (b[0] << 8) | b[1];
        idleTime = (b[2] << 8) | b[3];
        cst = (b[4] << 8) | b[5];
        rfl = (b[6] << 8) | b[7];
        lbt = b[9];
        cw = b[10];
        
       self.olOnOffTime.text = [NSString stringWithFormat:@"%d/%d",readTime,idleTime];
        [self.rcp getOutputPowerLevel];
    });
    
}

- (IBAction)fhLbtChanged:(id)sender {
    uint8_t fh;
    if ([self.olFh isOn]) {
        fh = 0x01;
    }else{
        fh = 0x00;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        BOOL success = [self.rcp setFhLbtParam:readTime
                                      idleTime:idleTime
                               carrierSenseTime:cst
                                        rfLevel:rfl
                               frequencyHopping:fh
                               listenBeforeTalk:lbt
                                 continuousWave:cw];
        NSString *msg = nil;
        if (success) {
            msg = @"設定完了";
        }else{
            msg = @"設定失敗";
        }
        
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:msg delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
        [alert show];

    });
}

- (IBAction)eTagRssiChanged:(id)sender
{
    [[NSUserDefaults standardUserDefaults] setBool:self.olTagRssi.isOn forKey:@"tagRssi"];    
    [[NSUserDefaults standardUserDefaults] synchronize];
}



@end
